==> tecfy_corev2_package-1.1.30.tar.gz: lib/models/entity/property_number.model.dart <==
part of tecfy_corev2_client_package;

class PropertyNumber extends Property {
  late double max;
  late double min;
  late String format;

  //-------------------------
  //------- Calculated Field
  String? calcFormula;
  late bool isCalced;
  late List<String> calcedProps;
  late String formula;
  late String formulaType;
  late math_expressions.Expression formulaExp;
  late List<math_expressions.Variable> formulaExpVariables;

  String? sumEntityName;
  String? sumPropertyName;

  PropertyNumber({
    required String dbName,
    required String nameAr,
    required String nameEn,
    bool isRequired = false,
    this.min = 0,
    this.max = double.infinity,
    bool readOnly = false,
    this.format = '',
    //required ValueTypes valueType
  }) : super(
            dbName: dbName,
            nameAr: nameAr,
            nameEn: nameEn,
            readOnly: readOnly,
            propertyType: PropertyTypes.Number,
            valueType: ValueTypes.Number,
            isRequired: isRequired);

  PropertyNumber.fromMap(Map<String, dynamic> row) : super.fromMap(row) {
    calcFormula = row['calcFormula'] ?? '';
    max = BaseModel.getDoubleEx(row['max']) ?? 0;
    min = BaseModel.getDoubleEx(row['min']) ?? 0;
    format = row['format'] ?? '';

    if (calcFormula!.isNotEmpty) {
      isCalced = true;
      var tempList = calcFormula!.split(',');
      formulaType = tempList[0];
      if (formulaType == 'Formula') {
        formula = tempList[1];
        calcedProps = tempList.sublist(2);
        var parser = math_expressions.Parser();
        formulaExp =
            parser.parse(formula.replaceAll('e', 'c')); //'(x^2 + cos(y)) / 3'
        formulaExpVariables = calcedProps
            .map((d) => math_expressions.Variable(d.replaceAll('e', 'c')))
            .toList();
      } else if (formulaType == 'SumSub') {
        sumEntityName = lowerFirst(tempList[1]);
        sumPropertyName = lowerFirst(tempList[2]);
      }
    } else
      isCalced = false;
  }

  /*
 keywords['nrt'] = TokenType.ROOT;
    keywords['sqrt'] = TokenType.SQRT;
    keywords['log'] = TokenType.LOG;
    keywords['cos'] = TokenType.COS;
    keywords['sin'] = TokenType.SIN;
    keywords['tan'] = TokenType.TAN;
    keywords['arccos'] = TokenType.ACOS;
    keywords['arcsin'] = TokenType.ASIN;
    keywords['arctan'] = TokenType.ATAN;
    keywords['abs'] = TokenType.ABS;
    keywords['ceil'] = TokenType.CEIL;
    keywords['floor'] = TokenType.FLOOR;
    keywords['sgn'] = TokenType.SGN;
    keywords['ln'] = TokenType.LN;
    keywords['e'] = TokenType.EFUNC;
     */

  double? getCalcResult(List<dynamic> vars) {
    var cm = math_expressions.ContextModel();
    for (int i = 0; i < vars.length; i++) {
      if (vars[i] == null ||
          !(vars[i] is num) ||
          (vars[i] as num).isInfinite ||
          (vars[i] as num).isNaN) vars[i] = 0;
      cm.bindVariable(
          formulaExpVariables[i], new math_expressions.Number(vars[i]));
    }
    // cm.bindVariable(y, new Number(Math.PI));
    // Evaluate expression:
    try {
      var eval = formulaExp.evaluate(math_expressions.EvaluationType.REAL, cm);
      if (!(eval is num) || eval.isNaN) eval = null;
      return eval;
    } catch (err) {
      print('xxxx Error calc resilt: $err');
      return null;
    }
  }

  @override
  OperationResult isValid(data) {
    if (format == '0' && data.toString().contains('.')) {
      return OperationResult(
          success: false, message: ' $nameAr يجب أن تكون رقم صحيح');
    }
    if (!super.isValid(data).success) return super.isValid(data);

    return OperationResult(success: true);
  }

  /// get the value and bind tp display widget
  Widget getDisplayWidget(Map<String, dynamic> data, {TextStyle? textStyle}) {
    String value = '';
    if (data[dbName] == null)
      value = '';
    else if (data[dbName] is int)
      value = data[dbName].toString();
    else if (format.isNotEmpty && data[dbName] != null && format.contains('.'))
      value =
          data[dbName]?.toStringAsFixed(format.split('.').last.length) ?? '';
    else
      value = data[dbName]?.toStringAsFixed(2) ?? '';

    return Text(value, style: textStyle);
  }

  /// get the value in text
  String getDisplayText(Map<String, dynamic> data) {
    String value = '';
    if (data[dbName] == null)
      value = '';
    else if (data[dbName] is int)
      value = data[dbName].toString();
    else if (format.isNotEmpty && data[dbName] != null && format.contains('.'))
      value =
          data[dbName]?.toStringAsFixed(format.split('.').last.length) ?? '';
    else
      value = data[dbName]?.toStringAsFixed(2) ?? '';

    return value;
  }

  Future<bool> getDefaultValue(Map<String, dynamic> data) async {
    if (data[dbName] != null) return false;
    if (defaultString != null && defaultString!.isNotEmpty) {
      data[dbName] = num.parse(defaultString!);
    }
    return false;
  }
}

==> tecfy_corev2_package-1.1.42.tar.gz: lib/models/entity/property_number.model.dart <==
part of tecfy_corev2_client_package;

class PropertyNumber extends Property {
  late double max;
  late double min;
  late String format;

  //-------------------------
  //------- Calculated Field
  String? calcFormula;
  late bool isCalced;
  late List<String> calcedProps;
  late String formula;
  late String formulaType;
  late math_expressions.Expression formulaExp;
  late List<math_expressions.Variable> formulaExpVariables;

  String? sumEntityName;
  String? sumPropertyName;

  PropertyNumber({
    required String dbName,
    required String nameAr,
    required String nameEn,
    bool isRequired = false,
    this.min = 0,
    this.max = double.infinity,
    bool readOnly = false,
    this.format = '',
    //required ValueTypes valueType
  }) : super(
            dbName: dbName,
            nameAr: nameAr,
            nameEn: nameEn,
            readOnly: readOnly,
            propertyType: PropertyTypes.Number,
            valueType: ValueTypes.Number,
            isRequired: isRequired);

  PropertyNumber.fromMap(Map<String, dynamic> row) : super.fromMap(row) {
    calcFormula = row['calcFormula'] ?? '';
    max = BaseModel.getDoubleEx(row['max']) ?? 0;
    min = BaseModel.getDoubleEx(row['min']) ?? 0;
    format = row['format'] ?? '';

    if (calcFormula!.isNotEmpty) {
      isCalced = true;
      var tempList = calcFormula!.split(',');
      formulaType = tempList[0];
      if (formulaType == 'Formula') {
        formula = tempList[1];
        calcedProps = tempList.sublist(2);
        var parser = math_expressions.Parser();
        formulaExp =
            parser.parse(formula.replaceAll('e', 'c')); //'(x^2 + cos(y)) / 3'
        formulaExpVariables = calcedProps
            .map((d) => math_expressions.Variable(d.replaceAll('e', 'c')))
            .toList();
      } else if (formulaType == 'SumSub') {
        sumEntityName = lowerFirst(tempList[1]);
        sumPropertyName = lowerFirst(tempList[2]);
      }
    } else
      isCalced = false;
  }

  /*
 keywords['nrt'] = TokenType.ROOT;
    keywords['sqrt'] = TokenType.SQRT;
    keywords['log'] = TokenType.LOG;
    keywords['cos'] = TokenType.COS;
    keywords['sin'] = TokenType.SIN;
    keywords['tan'] = TokenType.TAN;
    keywords['arccos'] = TokenType.ACOS;
    keywords['arcsin'] = TokenType.ASIN;
    keywords['arctan'] = TokenType.ATAN;
    keywords['abs'] = TokenType.ABS;
    keywords['ceil'] = TokenType.CEIL;
    keywords['floor'] = TokenType.FLOOR;
    keywords['sgn'] = TokenType.SGN;
    keywords['ln'] = TokenType.LN;
    keywords['e'] = TokenType.EFUNC;
     */

  double? getCalcResult(List<dynamic> vars) {
    var cm = math_expressions.ContextModel();
    for (int i = 0; i < vars.length; i++) {
      if (vars[i] == null ||
          !(vars[i] is num) ||
          (vars[i] as num).isInfinite ||
          (vars[i] as num).isNaN) vars[i] = 0;
      cm.bindVariable(
          formulaExpVariables[i], new math_expressions.Number(vars[i]));
    }
    // cm.bindVariable(y, new Number(Math.PI));
    // Evaluate expression:
    try {
      var eval = formulaExp.evaluate(math_expressions.EvaluationType.REAL, cm);
      if (!(eval is num) || eval.isNaN) eval = null;
      return eval;
    } catch (err) {
      print('xxxx Error calc resilt: $err');
      return null;
    }
  }

  @override
  OperationResult isValid(data) {
    if (format == '0' && data.toString().contains('.')) {
      return OperationResult(
          success: false, message: ' $nameAr يجب أن تكون رقم صحيح');
    }
    if (!super.isValid(data).success) return super.isValid(data);

    return OperationResult(success: true);
  }

  /// get the value and bind tp display widget
  Widget getDisplayWidget(Map<String, dynamic> data, {TextStyle? textStyle}) {
    // String value = '';
    // if (data[dbName] == null)
    //   value = '';
    // else if (data[dbName] is int)
    //   value = data[dbName].toString();
    // else if (format.isNotEmpty && data[dbName] != null && format.contains('.'))
    //   value =
    //       data[dbName]?.toStringAsFixed(format.split('.').last.length) ?? '';
    // else
    //   value = data[dbName]?.toStringAsFixed(2) ?? '';

    return Text(getDisplayText(data), style: textStyle);
  }

  /// get the value in text
  String getDisplayText(Map<String, dynamic> data) {
    String value = '';
    // if (dbName.contains('.')) {
    //   if (MapService.getValue(data, dbName) != null) {
    //     return MapService.getValue(data, dbName).toString();
    //   }
    // } else {
    if (MapService.getValue(data, dbName) == null)
      value = '';
    else if (MapService.getValue(data, dbName) is int)
      value = MapService.getValue(data, dbName).toString();
    else if (format.isNotEmpty &&
        MapService.getValue(data, dbName) != null &&
        format.contains('.'))
      value = MapService.getValue(data, dbName)
              ?.toStringAsFixed(format.split('.').last.length) ??
          '';
    else
      value = MapService.getValue(data, dbName)?.toStringAsFixed(2) ?? '';
    // }

    return value;
  }

  Future<bool> getDefaultValue(Map<String, dynamic> data) async {
    if (MapService.getValue(data, dbName) != null) return false;
    if (defaultString != null && defaultString!.isNotEmpty) {
      MapService.updateValue(data, dbName, num.parse(defaultString!));
      // data[dbName] = num.parse(defaultString!);
    }
    return false;
  }
}
